package com.examples.pfsoft.mispozosderiego;

import android.content.DialogInterface;
import android.content.res.Resources;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.Toast;

import com.google.firebase.database.DatabaseReference;

/**
 * Created by VRaul on 04/12/2016  - [Revision @PFSoft 08/2017]
 * Clase que gestiona la Actividad para Edicion de los Pozos
 */

public class ActivityEdicionPozo extends AppCompatActivity {

    private static int maxsectores;
    private long id;
    private long id_pozo;
    private InfoPozoRiego pozo;
    private InfoEstadoPozo estado_pozo;

    private EditText nombre;
    private EditText telefono;
    private Spinner numsectores;
    private CheckBox esgoteo;
    private EditText direccion;
    private EditText comentario;


    private Aplicacion app;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_edicionpozo);

        app = (Aplicacion) getApplication();

        //Gestíón Toolbar...
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar3);
        setSupportActionBar(toolbar);

    }


    @Override
    protected void onResume(){
        super.onResume();
        //Obtener id del Pozo a visualizar...
        Bundle extras = getIntent().getExtras();
        id = extras.getLong("id_edit", -1);
        id_pozo = extras.getLong("id_new", -1);

        if (id_pozo!=-1) { //Editar Nuevo Pozo....
            pozo = app.getPozos().getNewPozo();
            estado_pozo = app.getEstadopozos().getNewEstadoPozo();

        } else { //Editar Pozo existente
            if (id!=-1) {
                pozo = app.getPozos().elemento((int) id);
            } else {
                Log.d("EdicionPozo: ", "id no existe");
                Toast.makeText(this, "Edicion Pozo: id_pozo no existe", Toast.LENGTH_LONG).show();
                finish();
            }
        }

        maxsectores = pozo.getMAX_SECTORES();

        //Para acceder  por código a objetos del layout
        nombre = (EditText) findViewById(R.id.editNombre);
        telefono = (EditText) findViewById(R.id.editTelefono);
        esgoteo = (CheckBox)findViewById(R.id.editEsGoteo);
        direccion = (EditText) findViewById(R.id.editDireccion);
        comentario = (EditText) findViewById(R.id.editComentario);
        //Spinner para seleccionar el Número de sectores
        numsectores = (Spinner) findViewById(R.id.editNumSectores);
        ArrayAdapter<String> adaptador = new ArrayAdapter<String>(this,
                android.R.layout.simple_spinner_item, getListaSectores());
        adaptador.setDropDownViewResource(android.R.layout. simple_spinner_dropdown_item);
        numsectores.setAdapter(adaptador);

        //Llamada Procedimiento para rellenar EdicionPozo con la información del Pozo
        LoadEdicionPozo();

    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_editapozo, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.

        switch (item.getItemId()) {
            case R.id.accion_guardar:
                SaveEdicionPozo();
                return true;
            case R.id.accion_cancelar:
                //Distinguir cancelar cuando se añade uno nuevo
                //if (id_pozo!=-1) {
                    //app.getPozos().borrar((int) id_pozo);
                    //app.getEstadopozos().borrar((int) id_pozo);
                //}
                finish();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    void LoadEdicionPozo (){

        nombre.setText(pozo.getNombre());
        telefono.setText(pozo.getTelefono());
        numsectores.setSelection(pozo.getNum_sectores());
        esgoteo.setChecked(pozo.isEs_goteo());
        direccion.setText(pozo.getDireccion());
        comentario.setText(pozo.getComentario());
    }

    void SaveEdicionPozo (){

        Resources res = getResources();

         new AlertDialog.Builder(this)
            .setTitle(R.string.aldialog_03_Titulo)
            .setMessage(R.string.aldialog_03_Mensaje)
            .setPositiveButton(res.getString(R.string.txt_confirm), new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int whichButton) {
                    pozo.setNombre(nombre.getText().toString());
                    pozo.setTelefono(telefono.getText().toString());
                    pozo.setNum_sectores(numsectores.getSelectedItemPosition());
                    pozo.setEs_goteo(esgoteo.isChecked());
                    pozo.setDireccion(direccion.getText().toString());
                    pozo.setComentario(comentario.getText().toString());

                    if (id_pozo!=-1) { //Editar nuevo pozo/
                        DatabaseReference pozoRef = app.getPozosReference().child(app.getStrNextidpozo());
                        pozoRef.setValue(pozo);
                        DatabaseReference estadopozoRef = app.getEstadopozosReference().child(app.getStrNextidpozo());
                        estadopozoRef.setValue(estado_pozo);
                        // Actualizar next_idpozo...
                        app.getNext_idpozoReference().setValue(id_pozo + 1);
                    } else { //Editar Pozo existente
                        if (id != -1) {
                            app.getPozos().actualiza((int) id, pozo);
                        } else {
                            Log.d("EdicionPozo: ", "id no existe");
                            finish();
                        }
                    }

                    finish();
                }})
             .setNegativeButton(res.getString(R.string.txt_cancel), null)
             .show();

    }


    //Rellena Array String para el Spinner asociado al Numero Sectores...
    public static String[] getListaSectores(){
        int cont;
        String[] resultado = new String[(maxsectores + 1)];
        for (cont = 0; cont < resultado.length; cont++){
            resultado[cont] = Integer.toString(cont);
        }
        return resultado;
    }

}
