package com.examples.pfsoft.mispozosderiego;

import android.Manifest;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.content.res.Resources;
import android.os.Bundle;
import android.os.SystemClock;
import android.support.annotation.NonNull;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;

import com.firebase.ui.auth.AuthUI;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;

/**
 * Created by VRaul on 04/12/2016  - [Revision @PFSoft 08/2017]
 * Clase que gestiona la Actividad Principal: Mostrar Recycler View con Estado Actual de los Pozos
 */

public class ActivityMain extends AppCompatActivity {

    static final int RESULTADO_PREFERENCIAS = 1;
    private View vista;

    private RecyclerView recyclerView;
    private RecyclerView.LayoutManager layoutManager;
    private Aplicacion app;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        app = (Aplicacion) getApplication();
        vista = findViewById(R.id.activity_main);
        final Resources res = getResources();

        //Solicitar permisos de SMS para versión 6.0 y superiores...
        if (ContextCompat.checkSelfPermission(ActivityMain.this,
                Manifest.permission.RECEIVE_SMS) != PackageManager.PERMISSION_GRANTED) {
            solicitarPermiso(Manifest.permission.RECEIVE_SMS, res.getString(R.string.aldialog_05_Aviso),
                    GLOBALCONST.SOLICITUD_PERMISO_RECEIVE_SMS);
        }

        /*
        if (ContextCompat.checkSelfPermission(ActivityMain.this,
                Manifest.permission.READ_SMS) != PackageManager.PERMISSION_GRANTED) {
            solicitarPermiso(Manifest.permission.READ_SMS, res.getString(R.string.aldialog_05_Aviso),
                    GLOBALCONST.SOLICITUD_PERMISO_READ_SMS);
        }*/

        //Gestíón Toolbar...
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        //Escuchador de eventos para gestión Botón Flotante...
        FloatingActionButton fab = (FloatingActionButton) findViewById(R.id.fab);

        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                lanzarMapaPozos(null);
                Snackbar.make(view, "No implementado: Cambiará a Vista Mapa Pozos", Snackbar.LENGTH_LONG)
                        .setAction("Action", null).show();
            }
        });


         /* ===== Se hace en el servicio Principal ====================================
        //Asociar Receptores de Anuncios para recepción de SMS desde código...
        IntentFilter filtro = new IntentFilter(ReceptorSMS.ACTION_RECIBIR_SMS);
        filtro.addCategory(Intent.CATEGORY_DEFAULT);
        registerReceiver(new ReceptorSMS(), filtro);
        */

         //Arrancar el Servicio Principal - Gestiona el Rceptor de Anuncios para los SMS
        startService(new Intent(ActivityMain.this, ServicioPrincipal.class));

    }

    @Override
    protected void onResume(){
        super.onResume();

        //Codigo para Recycler View
        recyclerView = (RecyclerView) findViewById(R.id.recycler_view2);
        recyclerView.setAdapter(app.getAdaptador());
        layoutManager = new LinearLayoutManager(this);
        recyclerView.setLayoutManager(layoutManager);

        //Escuchador de eventos por selección de un elemento del Recycler View
        app.getAdaptador().setOnItemClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent i = new Intent(ActivityMain.this, ActivityVistaEstadoPozo.class);
                long idpos= (long) recyclerView.getChildAdapterPosition(v);
                //i.putExtra("id", idpos); //Funciona por posición en Recycler View
                //Funciona por id_pozo
                InfoPozoRiego pozo =  app.getPozos().elemento((int)idpos);
                i.putExtra("id", (long) pozo.getId_pozo());
                startActivity(i);
            }
        });
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.

        switch (item.getItemId()) {
            case R.id.accion_pozos:
                lanzarSelPozos(null);
                return true;
            case R.id.accion_preferencias:
                lanzarPreferencias(null);
                return true;
            case R.id.accion_logout:
                lanzarLogout(null);
                return true;
            case R.id.accion_stopservice:
                stopService(new Intent(ActivityMain.this, ServicioPrincipal.class));
                return true;
            case R.id.accion_acercade:
                lanzarAcercaDe(null);
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    /*
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == RESULTADO_PREFERENCIAS) {
            ActivitySeleccionPozo.adaptador.setCursor(ActivityMain.pozos.extraeCursor());
            ActivitySeleccionPozo.adaptador.notifyDataSetChanged();
        }
    }
    */


    //Abrir Actividad Acerca de...
    public void lanzarAcercaDe(View view){
        Intent i = new Intent(this, ActivityAcercaDe.class);
        startActivity(i);
    }

    //Ejecutar Logout...
    public void lanzarLogout(View view){
        AuthUI.getInstance().signOut(this)
                .addOnCompleteListener(new OnCompleteListener<Void>() {
                    @Override
                    public void onComplete(@NonNull Task<Void> task) {
                        SharedPreferences pref = getSharedPreferences(
                                "com.examples.pfsoft.mispozosderiego_internal", MODE_PRIVATE);
                        pref.edit().remove("provider").commit();
                        pref.edit().remove("email").commit();
                        pref.edit().remove("name").commit();
                        Intent i = new Intent(ActivityMain.this,ActivityLogin.class);
                        i.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP
                                | Intent.FLAG_ACTIVITY_NEW_TASK
                                | Intent.FLAG_ACTIVITY_CLEAR_TASK);
                        startActivity(i);
                        finish();
                    }
                });
    }

    //Abrir Actividad Seleccionar Pozos...
    public void lanzarSelPozos(View view){
        Intent i = new Intent(this, ActivitySeleccionPozo.class);
        startActivity(i);
    }

    //Abrir Actividad Preferencias...
    public void lanzarPreferencias(View view){
        Intent i = new Intent(this, ActivityPreferencias.class);
        //startActivityForResult(i, RESULTADO_PREFERENCIAS);
        startActivity(i);
    }

    //Abrir Actividad Vista Mapa Pozos...
    public void lanzarMapaPozos(View view){
        Intent i = new Intent(this, ActivityVistaMapaPozos.class);
        startActivity(i);
    }


    public void solicitarPermiso (String permiso, String justificacion ,int codigo ) {
        final String fpermis = permiso;
        final int fcodig= codigo;

        Resources res = getResources();

        if (ActivityCompat.shouldShowRequestPermissionRationale(this  ,permiso)) {
            new AlertDialog.Builder(this)
                    .setTitle(R.string.aldialog_05_Titulo)
                    .setMessage(justificacion)
                    .setPositiveButton(res.getString(R.string.txt_ok), new DialogInterface.OnClickListener() {
                        public void onClick (DialogInterface dialog, int whichButton) {
                            ActivityCompat.requestPermissions(ActivityMain.this ,new String[]{ fpermis }, fcodig );
                        }})
                    .show();
        } else {
            ActivityCompat.requestPermissions(this,  new String []{permiso }, codigo);
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {

        if (requestCode == GLOBALCONST.SOLICITUD_PERMISO_RECEIVE_SMS) {
            if (grantResults.length== 1 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                //Ejecutamos acción peligrosa

            } else {
                Snackbar.make(vista, R.string.aldialog_05_Aviso, Snackbar.LENGTH_LONG).show();
                SystemClock.sleep(2000);
                finish ();
            }
        }

        if (requestCode == GLOBALCONST.SOLICITUD_PERMISO_READ_SMS) {
            if (grantResults.length== 1 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                //Ejecutamos acción peligrosa

            } else {
                Snackbar.make(vista, R.string.aldialog_05_Aviso, Snackbar.LENGTH_LONG).show();
                SystemClock.sleep(2000);
                finish ();
            }
        }



    }

}
