package com.examples.pfsoft.mispozosderiego;

import android.Manifest;
import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.res.Resources;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.os.FileUriExposedException;
import android.provider.MediaStore;
import android.support.annotation.NonNull;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.CheckBox;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.storage.OnPausedListener;
import com.google.firebase.storage.OnProgressListener;
import com.google.firebase.storage.StorageReference;
import com.google.firebase.storage.UploadTask;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;


/**
 * Created by VRaul on 04/12/2016 - [Revision @PFSoft 08/2017]
 *
 * Clase que gestiona la Actividad para Mostrar la información del Pozo permitiendo su modificación
 * Se llama desde la Actividad SeleccionPozos al seleccionar un Pozo de la lista
 * Botón flotante para añadir pozo nuevo y Menú para editar y/o eliminar pozo existente y para
 * mostrar su ubicación en Google Maps
 * Botones sobre imagen para escogerla de la galería  y así poder asociarla al pozo
 * Las fotos seleccionadas de la galería se subirán a FirebaseStorage: Con un mombre fijo: pozo_<idpozo>.jpg
 * Botón sobre la imagen para poder eliminar foto :Actualizar campo asociado en Firebase - No elimina fichero de Firebase Storage
 *
 *
 * Nota:
 * Anteriormente las fotos asociadas a los pozos se almacenaban de forma local en el dispositivo móvil.
 */

public class ActivityVistaPozo extends AppCompatActivity {

    final static int RESULTADO_EDITAR = 1;
    final static int RESULTADO_GALERIA = 2;
    final static int RESULTADO_FOTO = 3;

    final int SOLICITUD_SUBIR_PUTFILE = 4;
    final int SOLICITUD_SELECCION_PUTFILE = 101;

    private ProgressDialog progresoSubida;
    Boolean subiendoDatos =false;

    private long id;
    private boolean esSelpozo;
    private InfoPozoRiego pozo;

    private View vista;
    private Context contexto;
    private ImageView imgLogoDireccion;
    private ImageView imgLogoGaleria;
    private ImageView imgLogoCamara;
    private ImageView imgLogoEliminarFoto;

    //Para gestionar imagen asociada al del Pozo
    private ImageView imageView;
    private Uri uriFoto;

    private Aplicacion app;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_vistapozo);

        vista = findViewById(R.id.activity_vistapozo);
        contexto = this;
        final Resources res = getResources();

        app = (Aplicacion) getApplication();

        //Gestíón Toolbar...
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar2);
        setSupportActionBar(toolbar);

        //Escuchador de eventos para gestión Botón Flotante...
        FloatingActionButton fab = (FloatingActionButton) findViewById(R.id.fab2);
        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                //----> Revisar Firebase ---> Botón Añadir nuevo pozo...
                long id_pozo = app.getPozos().nuevo();
                long id_pozo2 = app.getEstadopozos().nuevo((int) id_pozo);
                Intent i = new Intent(ActivityVistaPozo.this,ActivityEdicionPozo.class);
                i.putExtra("id_new",id_pozo);
                startActivity(i);

            }
        });

        //Escuchador de eventos para pulsación sobre icono logo_direccion...
        imgLogoDireccion =(ImageView) findViewById(R.id.logo_direccion);
        imgLogoDireccion.setOnClickListener(new View.OnClickListener() {
            public void onClick(View view) {
                verMapa(null);
            }
        });

        //Escuchador de eventos para pulsación sobre icono logo_galería...
        imgLogoGaleria =(ImageView) findViewById(R.id.logo_galeria);
        imgLogoGaleria.setOnClickListener(new View.OnClickListener() {
            public void onClick(View view) {
                if (ContextCompat.checkSelfPermission(ActivityVistaPozo.this,
                        Manifest.permission.READ_EXTERNAL_STORAGE) == PackageManager.PERMISSION_GRANTED) {
                    galeria(null);
                } else {
                    solicitarPermiso(Manifest.permission.READ_EXTERNAL_STORAGE, res.getString(R.string.aldialog_04_Aviso),
                            GLOBALCONST.SOLICITUD_PERMISO_READ_EXTERNAL_STORAGE);
                }
            }
        });

        //Escuchador de eventos para pulsación sobre icono logo_eliminar...
        imgLogoEliminarFoto =(ImageView) findViewById(R.id.logo_eliminarfoto);
        imgLogoEliminarFoto.setOnClickListener(new View.OnClickListener() {
            public void onClick(View view) {
                eliminarFoto(null);
            }
        });

        //Escuchador de eventos para pulsación sobre icono logo_cámara - Se ha eliminado esta funcionalidad
        /*
        imgLogoCamara =(ImageView) findViewById(R.id.logo_camara);
        imgLogoCamara.setOnClickListener(new View.OnClickListener() {
            public void onClick(View view) {
                tomarFoto(null);
            }
        });*/


    }



    @Override
    protected void onResume(){
        super.onResume();
        //Obtener id del Pozo a visualizar...
        Bundle extras = getIntent().getExtras();
        id = extras.getLong("id", -1);
        //=====>Firebase
        pozo = app.getPozos().elemento((int) id);
        //Llamada Procedimiento para rellenar VistaPozo con la información del Pozo
        SetVistaPozo();

    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_vistapozo, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.

        switch (item.getItemId()) {
            case R.id.accion_editar:
                lanzarEditaLugar(null);
                return true;
            case R.id.accion_borrar:
               //=====> Firebase...
                borrarPozo((int) id);
                return true;
            case R.id.accion_llegar:
                verMapa(null);
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    void SetVistaPozo (){
        Log.d("ID POZO",pozo.getNombre() + ": " + pozo.getId_pozo());
        TextView nombre = (TextView) findViewById(R.id.nompozo);
        nombre.setText(pozo.getNombre());
        TextView telefono = (TextView) findViewById(R.id.telefono);
        if (pozo.getTelefono().isEmpty()){
            telefono.setVisibility(View.GONE);
        } else {
            telefono.setVisibility(View.VISIBLE);
            telefono.setText(pozo.getTelefono());
        }
        TextView direccion = (TextView) findViewById(R.id.direccion);
        if (pozo.getDireccion().isEmpty()){
            direccion.setVisibility(View.GONE);
        } else {
            direccion.setVisibility(View.VISIBLE);
            direccion.setText(pozo.getDireccion());
        }
        TextView comentario = (TextView) findViewById(R.id.comentario);
        if (pozo.getComentario().isEmpty()){
            comentario.setVisibility(View.GONE);
        } else {
            comentario.setVisibility(View.VISIBLE);
            comentario.setText(pozo.getComentario());
        }
        TextView numsectores = (TextView) findViewById(R.id.numsectores);
        numsectores.setText(Integer.toString(pozo.getNum_sectores()));
        CheckBox esgoteo = (CheckBox)findViewById(R.id.es_goteo);
        esgoteo.setChecked(pozo.isEs_goteo());
        imageView = (ImageView) findViewById(R.id.foto);
        //Obtener foto desde URL usando un AsyTask
        new DownloadImageTask(imageView).execute(pozo.getFoto());

        //Version Antigua Obtener foto desde ALMACENAJE del Móvil - Se ha eliminado esta funcionalidad
        //ponerFoto(imageView, pozo.getFoto());
    }

    public void borrarPozo(final int id){
        Resources res = getResources();
        new AlertDialog.Builder(this)
            .setTitle(res.getString(R.string.aldialog_02_Titulo))
            .setMessage(res.getString(R.string.aldialog_02_Mensaje))
            .setPositiveButton(res.getString(R.string.txt_confirm), new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int whichButton) {
                    //====> Firebase =========================================================
                    app.getPozos().borrar(id); //Elimina pozo de la BD
                    app.getEstadopozos().borrar(id); ////Elimina estadopozo de la BD
                    //Eliminar también de la BD los riegos y alarmas asociados  asociados al pozo
                    InfoPozoRiego pozo_aux = app.getPozos().elemento(id);
                    String keyidpozo = app.getStrKeyIdPozo(pozo_aux.getId_pozo());
                    DatabaseReference riegosRef = app.getRiegosReference().child(keyidpozo);
                    riegosRef.removeValue();
                    DatabaseReference alarmasRef = app.getAlarmasReference().child(keyidpozo);
                    alarmasRef.removeValue();
                    //----> Revisar Firebase - Eliminar
                    //app.getAdaptadorpozos().notifyDataSetChanged();
                    finish();
                    }})
                .setNegativeButton(res.getString(R.string.txt_cancel), null)
                .show();
    }

    void actualizaPozo(){
        //====> Firebase ==================================================================
        int id_pozo = (int) id;
        app.getPozos().actualiza(id_pozo, pozo);
        //----> Revisar Firebase - Eliminar
        //app.getAdaptadorpozos().notifyItemChanged((int) id);
    }

    //Acción Boton Editar Pozo: Abrir Actividad Editar Pozos...
    public void lanzarEditaLugar(View view){
        Intent i = new Intent(this, ActivityEdicionPozo.class);
        i.putExtra("id_edit", id);
        startActivity(i);
    }

    //Acción Menu e icono dirección:
    //Abrir Actividad con Mapa de Google para editar geolocalización...
    public void verMapa(View view){
        Intent i = new Intent(this, ActivityVistaMapaEditar.class);
        i.putExtra("id_mapa", id);
        startActivity(i);
    }

    /*
    // Llamar Intención implicita paraMostrar alicacion Mapas para ver localización Pozo
    public void verMapa(View view){
        Uri uri;

        //Toast.makeText(this, "Abrir aplicación Mapas", Toast.LENGTH_LONG).show();

        double lat = pozo.getPosicion().getLatitud();
        double lon = pozo.getPosicion().getLongitud();

        if (lat != 0 || lon != 0) {
            Log.d("Posicion Mapa ", "Latitud: " + Double.toString(lat) + "  Longitud: "+ Double.toString(lon));
            uri = Uri.parse("geo:" + lat + "," + lon);
        } else {
            Log.d("Posicion Mapa ", "Direccion: " + pozo.getDireccion());
            uri = Uri.parse("geo:0,0?q=" + pozo.getDireccion());
        }
        Intent intent = new Intent(Intent.ACTION_VIEW, uri);
        startActivity(intent);

    }
    */




    //Acción Botón Galería: Llamar a Intención para obtener foto de la galería de imágenes
    public void galeria(View view) {

        //Obtener imagen de Galeria - Versión para Firebase Storage
        Intent seleccionFotografiaIntent = new Intent(Intent.ACTION_PICK);
        seleccionFotografiaIntent.setType("image/*");
        startActivityForResult(seleccionFotografiaIntent, RESULTADO_GALERIA );

        /*
        //Obtener imagen de Galeria - Versión Antigua Almacenaje en Móvil - Se ha eliminado esta funcionalidad
        Intent intent = new Intent(Intent.ACTION_PICK, MediaStore.Images.Media.EXTERNAL_CONTENT_URI);
        startActivityForResult(intent, RESULTADO_GALERIA);
        */
    }


    //Acción Botón Eliminar Foto: Eliminar imagen asociada pozo
    public void eliminarFoto(View view) {
        pozo.setFoto(null);
        imageView.setImageBitmap(null);
        actualizaPozo();
        //Eliminar de Firebase Storage el fichero...

    }


    /*
    //Llamada intención para hacer una foto con la cámara del dispositivo - Se ha eliminado esta funcionalidad
    public void tomarFoto(View view) {
        Intent intent = new Intent("android.media.action.IMAGE_CAPTURE");
        uriFoto = Uri.fromFile(new File( Environment.getExternalStorageDirectory() +
                File.separator + "img_" + (System.currentTimeMillis() / 1000) + ".jpg"));
        intent.putExtra(MediaStore.EXTRA_OUTPUT, uriFoto);
        startActivityForResult(intent, RESULTADO_FOTO);
    }


    //Poner imagen del Pozo - Versión Antigua Almacenaje en dispositivo Móvil - Se ha eliminado esta funcionalidad
    protected void ponerFoto(ImageView imageView, String uri) {
        if (uri != null) {
            //Reduce la imagen para evitar errores de memoria con fotos grandes
            imageView.setImageBitmap(reduceBitmap(this, uri, 2048, 2048));;
        } else {
            imageView.setImageBitmap(null);
        }
    }


    //Reduce la imagen para evitar problemas - Versión Antigua Almacenaje en dispositivo Móvil - Se ha eliminado esta funcionalidad
    public static Bitmap reduceBitmap(Context contexto, String uri,
                                      int maxAncho, int maxAlto) {
        try {
            final BitmapFactory.Options options = new BitmapFactory.Options();
            options.inJustDecodeBounds = true;
            BitmapFactory.decodeStream(contexto.getContentResolver()
                    .openInputStream(Uri.parse(uri)), null, options);
            options.inSampleSize = (int) Math.max(
                    Math.ceil(options.outWidth / maxAncho),
                    Math.ceil(options.outHeight / maxAlto));
            options.inJustDecodeBounds = false;
            return BitmapFactory.decodeStream(contexto.getContentResolver()
                    .openInputStream(Uri.parse(uri)), null, options);
        } catch (FileNotFoundException e) {
            Toast.makeText(contexto, "Fichero/recurso no encontrado", Toast.LENGTH_LONG).show();
            e.printStackTrace();
            return null;
        }
    }
    */

    //Versión Nueva de Fotos para Firebase Storage
    public void subirAFirebaseStorage(String ficheroDispositivo) {

        final ProgressDialog progresoSubida =
                ProgressDialog.show(ActivityVistaPozo.this, "Espere ...", "Subiendo ...", true);
        UploadTask uploadTask=null;
        String fichero = "pozo_" + Integer.toString(pozo.getId_pozo()) + ".jpg";
        StorageReference imagenRef = app.getStorageReference().child(fichero);

        try {
            Uri file = Uri.fromFile(new File(ficheroDispositivo));
            uploadTask = imagenRef.putFile(file);

            uploadTask.addOnFailureListener(new OnFailureListener() {
                @Override
                public void onFailure(@NonNull Exception exception) {
                    progresoSubida.dismiss();
                    app.mostrarDialogo(app.getApplicationContext(), "Ha ocurrido un error al subir la imagen.");
                }
            }).addOnSuccessListener(new OnSuccessListener<UploadTask.TaskSnapshot>() {
                @Override
                public void onSuccess(UploadTask.TaskSnapshot taskSnapshot) {
                    String urlfoto = taskSnapshot.getDownloadUrl().toString();
                    pozo.setFoto(urlfoto);
                    actualizaPozo();
                    //Actualiza imagen pozo en la Actividad de la Vista del Pozo
                    new DownloadImageTask((ImageView) imageView).execute(urlfoto);
                    progresoSubida.dismiss();
                    subiendoDatos = false;
                    app.mostrarDialogo(app.getApplicationContext(), "Imagen subida correctamente.");
                }
            }).addOnProgressListener(new OnProgressListener<UploadTask.TaskSnapshot>() {
                @Override
                public void onProgress(UploadTask.TaskSnapshot taskSnapshot) {
                    if (!subiendoDatos) {
                        progresoSubida.show();
                        subiendoDatos = true;
                    }
                }
            }).addOnPausedListener(new OnPausedListener<UploadTask.TaskSnapshot>() {
                @Override
                public void onPaused(UploadTask.TaskSnapshot taskSnapshot) {
                    progresoSubida.dismiss();
                    subiendoDatos = true;
                }
            });
        } catch (Exception e){
            app.mostrarDialogo(app.getApplicationContext(), "Fallo en subida...");
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {

        //Versión Nueva de Fotos para Firebase Storage
        Uri ficheroSeleccionado;
        Cursor cursor;
        String rutaImagen;

        if (resultCode == Activity.RESULT_OK) {
            if (requestCode == RESULTADO_GALERIA) {
                ficheroSeleccionado = data.getData();
                String[] proyeccionFile = {MediaStore.Images.Media.DATA};
                cursor = getContentResolver().query(ficheroSeleccionado, proyeccionFile, null, null, null);
                cursor.moveToFirst();
                rutaImagen = cursor.getString(cursor.getColumnIndex(proyeccionFile[0]));
                cursor.close();
                subirAFirebaseStorage(rutaImagen);
            } else if ((requestCode == RESULTADO_FOTO)&& (pozo!=null) && (uriFoto!=null))  {
                rutaImagen = uriFoto.toString();
                subirAFirebaseStorage(rutaImagen);
            }
        }

        /*
        //Versión Antigua de Fotos utilizando Almacenaje en Móvil - Se ha eliminado esta funcionalidad
        if (requestCode == RESULTADO_GALERIA && resultCode == Activity.RESULT_OK) {
            Log.d(" ====> Uri galeria ", data.getDataString() + "<=====");
            pozo.setFoto(data.getDataString());
            //ponerFoto(imageView, pozo.getFoto());

            actualizaPozo();
        } else if (requestCode == RESULTADO_FOTO && resultCode == Activity.RESULT_OK
                && pozo!=null && uriFoto!=null) {
            Log.d(" ====> Uri Camara ", data.getDataString() + "<=====");
            pozo.setFoto(uriFoto.toString());

            //ponerFoto(imageView, pozo.getFoto());
            actualizaPozo();
        }
        */
    }


    //Gestion Solicitud Permisos para acceder a Memoria Externa del dispositivo

    public void solicitarPermiso (String permiso, String justificacion ,int codigo ) {
        final String fpermis = permiso;
        final int fcodig= codigo;

        Resources res = getResources();

        if (ActivityCompat.shouldShowRequestPermissionRationale(this  ,permiso)) {
            new AlertDialog.Builder(this)
                    .setTitle(R.string.aldialog_04_Titulo)
                    .setMessage(justificacion)
                    .setPositiveButton(res.getString(R.string.txt_ok), new DialogInterface.OnClickListener() {
                        public void onClick (DialogInterface dialog, int whichButton) {
                            ActivityCompat.requestPermissions(ActivityVistaPozo.this ,new String[]{ fpermis }, fcodig );
                        }})
                    .show();
        } else {
            ActivityCompat.requestPermissions(this,  new String []{permiso }, codigo);
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {

        if (requestCode == GLOBALCONST.SOLICITUD_PERMISO_READ_EXTERNAL_STORAGE) {
            if (grantResults.length== 1 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                //Ejecutamos acción peligrosa
                galeria(null);
            } else {
                Snackbar.make(vista, R.string.aldialog_04_Aviso,
                        Snackbar.LENGTH_SHORT).show();
            }
        }


    }



}



