package com.examples.pfsoft.mispozosderiego;

import android.content.Context;
import android.content.SharedPreferences;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.preference.PreferenceManager;
import android.util.Log;

import com.google.firebase.database.ChildEventListener;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;

import java.text.DecimalFormat;
import java.util.ArrayList;

/**
 * Created by VRaul on 15/08/2016 - @PFSoft 2017
 * Clase que implementa Interfaz para Almacenar Pozos  - - Basado en BD Firebase
 *
 */

public class AlmacenInfoEstadoFBase implements AlmacenInfoEstado, ChildEventListener {

    Context contexto;
    private Aplicacion app;

    //Usamos SDK Firebase para tener más control - Necesitamos una estructura que almacene los datos.
    private ArrayList<String> keys;
    private ArrayList<DataSnapshot> items;

    private InfoEstadoPozo newEstadoPozo; //Auxiliar nuevo Estado Pozo

        public AlmacenInfoEstadoFBase(Context contexto) {
        this.contexto = contexto;
        this.app = (Aplicacion) contexto.getApplicationContext();

        keys = new ArrayList<String>();
        items = new ArrayList<DataSnapshot>();

        app.getEstadopozosReference().addChildEventListener(this); //Escuchador asociado a cambios en los estados de los pozos...
    }

    public InfoEstadoPozo getNewEstadoPozo() {
        return newEstadoPozo;
    }

    public void setNewEstadoPozo(InfoEstadoPozo newEstadoPozo) {
        this.newEstadoPozo = newEstadoPozo;
    }

    //Referencia de cada nodo de la lista
    public DatabaseReference getRef(int id) {
        return items.get(id).getRef();
    }

    //Obtener Clave asociada a un Id
    public String getItemKey(int id) {
        return keys.get(id);
    }

    //Obtener Clave asociada a partir de un ID_POZO
    public String getStrKeyIdPozo(int idpozo) {
        DecimalFormat fmt = new DecimalFormat("00");
        String key = 'P' + fmt.format(idpozo);
        Log.d("Key - IDPOZO: ", key);
        return key;
    }


    @Override
    public InfoEstadoPozo elemento(int id){
        InfoEstadoPozo estadopozo = items.get(id).getValue(InfoEstadoPozo.class);
        return estadopozo;
    }

    //Obtener elemento a partir de la Clave asociada ID_POZO
    public InfoEstadoPozo elementoByKey(String key){
        int index = keys.indexOf(key); //Obtiene indice lista donde aparece la Clave
        if (index != -1) {
            InfoEstadoPozo estadopozo = items.get(index).getValue(InfoEstadoPozo.class);
            return estadopozo;
        } else {
            return null;
        }
    }

    @Override
    public void anyade(InfoEstadoPozo estadopozo) {

    }

    @Override
    public int nuevo(int id) {

        int id_pozo = app.getNext_idpozo();
        newEstadoPozo = new InfoEstadoPozo(id_pozo);

        return id_pozo;
    }

    @Override
    public void borrar(int id) {
        //Obtiene la referencia al estado del pozo y actualiza su contenido
        DatabaseReference estadopozoRef = app.getEstadopozosReference().child(getItemKey(id));
        estadopozoRef.removeValue();
    }

    @Override
    public int tamanyo() {
        return items.size();
    }

    @Override
    public void actualiza(int id, InfoEstadoPozo estadopozo) {
        //Obtiene la referencia al estado del pozo y actualiza su contenido
        DatabaseReference estadopozoRef = app.getEstadopozosReference().child(getItemKey(id));
        estadopozoRef.setValue(estadopozo);
    }

    //Obtener elemento a partir de la Clave asociada ID_POZO
    public void actualizaByKey(String key, InfoEstadoPozo estadopozo ){
        int index = keys.indexOf(key); //Obtiene indice lista donde aparece la Clave
        if (index != -1) {
            //Obtiene la referencia al estado del pozo y actualiza su contenido
            DatabaseReference estadopozoRef = app.getEstadopozosReference().child(getItemKey(index));
            estadopozoRef.setValue(estadopozo);
        }

    }

    /*
    public static InfoEstadoPozo extraeEstadoPozo(Cursor cursor) {

        InfoEstadoPozo estadopozo = new InfoEstadoPozo();

        estadopozo.setFecha_hora(cursor.getLong(0));
        estadopozo.setFecha_inicio(cursor.getString(1));
        estadopozo.setHora_inicio(cursor.getString(2));
        estadopozo.setPrograma(cursor.getInt(3));
        estadopozo.setSectores(cursor.getInt(4));
        estadopozo.setDuracion(cursor.getInt(5));
        estadopozo.setCaudal(cursor.getFloat(6));
        estadopozo.setPot_consumida(cursor.getFloat(7));
        estadopozo.setFechahora_sms(cursor.getLong(8));
        estadopozo.setEstado_pozo(ETipoEstado.values()[cursor.getInt(9)]);
        estadopozo.setInfo_alarma(ETipoAlarma.values()[cursor.getInt(10)]);
        estadopozo.setId_riego(cursor.getInt(11));
        estadopozo.setId_pozo(cursor.getInt(12));

        return estadopozo;
    }
   */

    @Override public void onChildAdded(DataSnapshot dataSnapshot, String s) {

        Log.d("OnChildAdded Estados: ", Integer.toString(dataSnapshot.getValue(InfoEstadoPozo.class).getId_pozo()));
        items.add(dataSnapshot);
        Log.d("Key - Estados: ", dataSnapshot.getKey());
        keys.add(dataSnapshot.getKey());
        app.getAdaptador().notifyItemInserted(app.getAdaptador().getItemCount()-1);
    }
    @Override public void onChildChanged(DataSnapshot dataSnapshot, String s) {
        String key = dataSnapshot.getKey();
        int index = keys.indexOf(key);
        if (index!=-1) {
            items.set(index, dataSnapshot);
            app.getAdaptador().notifyItemChanged(index, dataSnapshot.getValue(InfoEstadoPozo.class));
        }
    }
    @Override public void onChildRemoved(DataSnapshot dataSnapshot) {
        String key = dataSnapshot.getKey();
        int index = keys.indexOf(key);
        if (index!=-1) {
            keys.remove(index);
            items.remove(index);
            app.getAdaptador().notifyItemRemoved(index);
        }
    }
    @Override public void onChildMoved(DataSnapshot dataSnapshot, String s) {}
    @Override public void onCancelled(DatabaseError databaseError) {}


    public void activaEscuchadorLibros(){
        keys = new ArrayList<String>();
        items = new ArrayList<DataSnapshot>();
        app.getEstadopozosReference().addChildEventListener(this);
    }

    public void desactivaEscuchadorLibros(){
        app.getEstadopozosReference().removeEventListener(this);
    }


}


