package com.examples.pfsoft.mispozosderiego;

import android.content.Context;
import android.util.Log;

import com.google.firebase.database.ChildEventListener;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;

import java.util.ArrayList;

/**
 * Created by VRaul on 15/08/2016 - @PFSoft 2017
 * Clase que implementa Interfaz para Almacenar Pozos  - Basado en BD Firebase
 *
 */

public class AlmacenPozosFBase implements AlmacenPozos, ChildEventListener {

    private Context contexto;
    private Aplicacion app;

    //Usamos SDK Firebase para tener más control - Necesitamos una estructura que almacene los datos.
    private ArrayList<String> keys;
    private ArrayList<DataSnapshot> items;

    private InfoPozoRiego newPozo; //Auxiliar nuevo Pozo


    public AlmacenPozosFBase(Context contexto) {
        this.contexto = contexto;
        this.app = (Aplicacion) contexto.getApplicationContext();

        keys = new ArrayList<String>();
        items = new ArrayList<DataSnapshot>();

        app.getPozosReference().addChildEventListener(this); //Escuchador asociado a cambios en los pozos...

    }

    public InfoPozoRiego getNewPozo() {
        return newPozo;
    }

    public void setNewPozo(InfoPozoRiego newPozo) {
        this.newPozo = newPozo;
    }

    //Referencia de cada nodo de la lista
    public DatabaseReference getRef(int id) {
        return items.get(id).getRef();
    }


    //Obtener Clave asociada a un Id
    public String getItemKey(int id) {
        return keys.get(id);
    }



    @Override
    public InfoPozoRiego elemento(int id) {
        return items.get(id).getValue(InfoPozoRiego.class);
    }

    //Obtener elemento a partir de la Clave asociada ID_POZO
    public InfoPozoRiego elementoByKey(String key){
        int index = keys.indexOf(key); //Obtiene indice lista donde aparece la Clave
        if (index != -1) {
            return items.get(index).getValue(InfoPozoRiego.class);
        } else {
            return null;
        }
    }

    @Override
    public void anyade(InfoPozoRiego pozo) {

    }

    @Override
    public int nuevo() {
        int id_pozo = app.getNext_idpozo();
        newPozo = new InfoPozoRiego(id_pozo);

        return id_pozo;
    }

    @Override
    public void borrar(int id) {
        //Obtiene la referencia al pozo y actualiza su contenido
        DatabaseReference pozoRef = app.getPozosReference().child(getItemKey(id));
        pozoRef.removeValue();
    }

    @Override
    public int tamanyo() {
        return items.size();
    }

    @Override
    public void actualiza(int id, InfoPozoRiego pozo) {
       //Obtiene la referencia al pozo y actualiza su contenido
        DatabaseReference pozoRef = app.getPozosReference().child(getItemKey(id));
        pozoRef.setValue(pozo);
    }

    /*
    public static InfoPozoRiego extraePozo(Cursor cursor) {

        InfoPozoRiego pozo = new InfoPozoRiego();

        pozo.setId_pozo(cursor.getInt(0));
        pozo.setNombre(cursor.getString(1));
        pozo.setDireccion(cursor.getString(2));
        pozo.setPosicion(new InfoGeoPunto(cursor.getDouble(3), cursor.getDouble(4)));
        pozo.setTelefono(cursor.getString(5));
        pozo.setEs_goteo(Boolean.valueOf(cursor.getString(6)));
        Log.d(cursor.getString(1) + "- EsGoteo: ", cursor.getString(6) + "/" + Boolean.valueOf(cursor.getString(6)));
        pozo.setNum_sectores(cursor.getInt(7));
        pozo.setComentario(cursor.getString(8));
        pozo.setFoto(cursor.getString(9));
        pozo.setId_propietario(cursor.getInt(10));

        return pozo;
    }
*/

    @Override public void onChildAdded(DataSnapshot dataSnapshot, String s) {
        Log.d("OnChildAdded", dataSnapshot.getKey());
        items.add(dataSnapshot);
        keys.add(dataSnapshot.getKey());
        Log.d("Key: ", dataSnapshot.getKey());
        app.getAdaptadorpozos().notifyItemInserted(app.getAdaptadorpozos().getItemCount()-1);
    }
    @Override public void onChildChanged(DataSnapshot dataSnapshot, String s) {
        String key = dataSnapshot.getKey();
        int index = keys.indexOf(key);
        if (index!=-1) {
            items.set(index, dataSnapshot);
            app.getAdaptadorpozos().notifyItemChanged(index, dataSnapshot.getValue(InfoPozoRiego.class));
        }
    }
    @Override public void onChildRemoved(DataSnapshot dataSnapshot) {
        String key = dataSnapshot.getKey();
        int index = keys.indexOf(key);
        if (index!=-1) {
            keys.remove(index);
            items.remove(index);
            app.getAdaptadorpozos().notifyItemRemoved(index);
        }
    }
    @Override public void onChildMoved(DataSnapshot dataSnapshot, String s) {}
    @Override public void onCancelled(DatabaseError databaseError) {}


    public void activaEscuchadorLibros(){
        keys = new ArrayList<String>();
        items = new ArrayList<DataSnapshot>();
        app.getPozosReference().addChildEventListener(this);
    }
    public void desactivaEscuchadorLibros(){
        app.getPozosReference().removeEventListener(this);
    }



}


