package com.examples.pfsoft.mispozosderiego;

import android.app.Application;
import android.content.Context;
import android.content.Intent;
import android.util.Log;

import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import com.google.firebase.storage.FirebaseStorage;
import com.google.firebase.storage.StorageReference;

import java.text.DecimalFormat;

/**
 * Created by VRaul on 09/08/2017.
 */

public class Aplicacion extends Application {

    private final static String NEXTIDPOZO_CHILD = "next_idpozo";
    private final static String POZOS_CHILD = "pozos";
    private final static String ESTADOPOZOS_CHILD = "estadopozos";
    private final static String RIEGOS_CHILD = "riegos";
    private final static String ALARMAS_CHILD = "alarmas";

    private final static String FBSTORAGE_URL= "gs://mispozosderiego.appspot.com";
    private final static String FBSTORAGE_CHILD= "imgpozos";

    private DatabaseReference next_idpozoReference;
    private DatabaseReference pozosReference;
    private DatabaseReference estadopozosReference;
    private DatabaseReference riegosReference;
    private DatabaseReference alarmasReference;


    private AlmacenPozosFBase pozos;
    private AlmacenInfoEstadoFBase estadopozos;
    private AlmacenRiegoHistFBase riegos;
    private AlmacenAlarmaHistFBase alarmas;


    private AdaptadorEstadoPozos adaptador;
    private AdaptadorPozos adaptadorpozos;
    private AdaptadorRiegoHist adaptadorRiegos;
    private AdaptadorAlarmaHist adaptadorAlarmas;

    private int next_idpozo;

    private FirebaseStorage storage;
    private static StorageReference storageRef;
    private static Context context;

    private FirebaseAuth auth;

    @Override
    public void onCreate() {
        super.onCreate();

        //Contexto de la Aplicacion
        Aplicacion.context = getApplicationContext();

        //Para Autenticación en Firebase
        auth = FirebaseAuth.getInstance();
        //Referencias a la BD Firebase....
        FirebaseDatabase bdfirebase = FirebaseDatabase.getInstance(); //Obtener instancia de la BD
        bdfirebase.setPersistenceEnabled(true);

        pozosReference = bdfirebase.getReference().child(POZOS_CHILD); //Referencia a los pozos
        estadopozosReference = bdfirebase.getReference().child(ESTADOPOZOS_CHILD); //Referencia a los estados de los pozos
        riegosReference = bdfirebase.getReference().child(RIEGOS_CHILD); //Referencia a los riegos
        alarmasReference = bdfirebase.getReference().child(ALARMAS_CHILD); //Referencia a las alarmas

        //Referencia al elemento que nos indica el proximo valor del id_pozo a insertar...
        //Y su escuchador asociado para detectar cambios en su valor y actualizarse en consecuencia
        next_idpozoReference = bdfirebase.getReference().child(NEXTIDPOZO_CHILD);
        next_idpozoReference.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                int value = dataSnapshot.getValue(Integer.class);
                Log.d("Ejemplo Firebase", "Valor: " + value);
                setNext_idpozo(value);
            }
            @Override
            public void onCancelled(DatabaseError error) {
                Log.w("Ejemplo Firebase", "Error al leer.", error.toException());
            }
        });

        pozos = new AlmacenPozosFBase(this); //Utiliza BD Firebase
        estadopozos = new AlmacenInfoEstadoFBase(this); //Utiliza BD Firebase
        adaptador = new AdaptadorEstadoPozos(this, estadopozos, pozos);
        adaptadorpozos = new AdaptadorPozos(this, pozos);

        //Referencias al FirebaseStorage para las imagenes de los pozos
        storage = FirebaseStorage.getInstance();
        storageRef = storage.getReferenceFromUrl(FBSTORAGE_URL).child(FBSTORAGE_CHILD);

    }

    public FirebaseAuth getAuth() {
        return auth;
    }

    public static StorageReference getStorageReference() {
        return storageRef;
    }

    public AlmacenInfoEstadoFBase getEstadopozos() {
        return estadopozos;
    }

    public AlmacenPozosFBase getPozos() {
        return pozos;
    }

    public void setRiegos(AlmacenRiegoHistFBase riegos) {
        this.riegos = riegos;
    }

    public AlmacenRiegoHistFBase getRiegos() {
        return riegos;
    }

    public void setAlarmas(AlmacenAlarmaHistFBase alarmas) {
        this.alarmas = alarmas;
    }

    public AlmacenAlarmaHistFBase getAlarmas() {
        return alarmas;
    }

    public AdaptadorEstadoPozos getAdaptador() {
        return adaptador;
    }

    public AdaptadorPozos getAdaptadorpozos() {
        return adaptadorpozos;
    }

    public void setAdaptadorRiegos(AdaptadorRiegoHist adaptadorRiegos) {
        this.adaptadorRiegos = adaptadorRiegos;
    }

    public AdaptadorRiegoHist getAdaptadorRiegos() {
        return adaptadorRiegos;
    }

    public void setAdaptadorAlarmas(AdaptadorAlarmaHist adaptadorAlarmas) {
        this.adaptadorAlarmas = adaptadorAlarmas;
    }

    public AdaptadorAlarmaHist getAdaptadorAlarmas() {
        return adaptadorAlarmas;
    }

    //Referencia principal al nodo pozos
    public DatabaseReference getPozosReference() {
        return pozosReference;
    }

    public DatabaseReference getNext_idpozoReference() {
        return next_idpozoReference;
    }

    //Referencia principal al nodo estadopozos
    public DatabaseReference getEstadopozosReference() {
        return estadopozosReference;
    }

    public DatabaseReference getRiegosReference() {
        return riegosReference;
    }

    public DatabaseReference getAlarmasReference() {
        return alarmasReference;
    }

    public int getNext_idpozo() {
        return next_idpozo;
    }

    public void setNext_idpozo(int next_idpozo) {
        this.next_idpozo = next_idpozo;
    }

    //Referencia Next_idPozo como String
    public String getStrNextidpozo(){
        DecimalFormat fmt = new DecimalFormat("00");
        String key = 'P' + fmt.format(next_idpozo);
        Log.d("Key - NEXT_IDPOZO: ", key);
        return key;
    }

    //Obtener Clave asociada a partir de un ID_POZO
    public String getStrKeyIdPozo(int idpozo) {
        DecimalFormat fmt = new DecimalFormat("00");
        String key = 'P' + fmt.format(idpozo);
        Log.d("Key - IDPOZO: ", key);
        return key;
    }

    //Obtener Clave asociada a partir de un ID_RIEGO
    public String getStrKeyIdRiego(int idriego) {
        DecimalFormat fmt = new DecimalFormat("000000");
        String key = 'R' + fmt.format(idriego);
        Log.d("Key - IDRIEGO: ", key);
        return key;
    }

    //Obtener Clave asociada a partir de un ID_ALARMA
    public String getStrKeyIdAlarma(int idalarma) {
        DecimalFormat fmt = new DecimalFormat("000000");
        String key = 'A' + fmt.format(idalarma);
        Log.d("Key - IDALARMA: ", key);
        return key;
    }

    static void mostrarDialogo (final Context context, final String mensaje){
        Intent intent = new Intent(context, ActivityDialogo.class);
        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        intent.putExtra("mensaje", mensaje);
        context.startActivity(intent);
    }

}

