package com.examples.pfsoft.mispozosderiego;

/**
 * Created by VRaul on 11/09/2017 - @PFSoft 2017
 * Clase que gestiona la Información del Histórico de Alarmas
 */

public class InfoAlarmaHist {
    private int id_pozo; //Clave relacional para enlaza con la información de Tabla Pozos
    private int id_riego; //Clave relacional con Tabla Historico Riegos (0: Sin riego asociado)
    private int id_alarma; //Clave relacional con Tabla Historico Riegos (0: Sin riego asociado)
    private long fecha_hora;  //Fecha y Hora Registro en BD en milisegundos  [Formato visualización:dd/mm/aaaa - hh:mm]
    private String fecha_inicio; //Fecha Inicio Riego o Alarma [Formato visualización:dd/mm/aaaa]
    private String hora_inicio;  //Hora Inicio Riego o Alarma [Formato visualización:hh:mm]
    private int duracion; //Duración  de la alarma hasta su rearme (expresado en minutos)  [Formato visualización:hh:mm]
    private long fechahora_sms;  //Fecha y Hora Recepcion SMS Actualización InfoEstadoRiego en milisegundos  [Formato visualización:dd/mm/aaaa - hh:mm]
    private int estado_pozo; //Ordinal asociada a los posibles Estados del Pozo
    private int info_alarma; //Ordinal asociada al Estado de Alarma del Pozo


    //Constructores

    public InfoAlarmaHist(){
        id_pozo = 0;
        fecha_hora= System.currentTimeMillis();
        id_riego = 0;
        id_alarma = 0;
        fecha_inicio = "";
        hora_inicio = "";
        duracion = 0;
        fechahora_sms = System.currentTimeMillis();
        estado_pozo = ETipoEstado.NINGUNO.ordinal();
        info_alarma = ETipoAlarma.NINGUNO.ordinal();

    }

    public InfoAlarmaHist(int id_pozo, int id_riego){
        this.id_pozo = id_pozo;
        this.id_riego  = id_riego;
        id_alarma = 0;
        fecha_hora = System.currentTimeMillis();
        fecha_inicio = "";
        hora_inicio = "";
        duracion = 0;
        fechahora_sms = System.currentTimeMillis();
        estado_pozo = ETipoEstado.NINGUNO.ordinal();
        info_alarma = ETipoAlarma.NINGUNO.ordinal();
    }

    public InfoAlarmaHist(int id_pozo, int id_riego, int id_alarma, long fecha_hora, String fecha_inicio, String hora_inicio, int duracion, long fechahora_sms, int estado_pozo, int info_alarma) {
        this.id_pozo = id_pozo;
        this.id_alarma = id_alarma;
        this.id_riego = id_riego;
        this.fecha_hora = fecha_hora;
        this.fecha_inicio = fecha_inicio;
        this.hora_inicio = hora_inicio;
        this.duracion = duracion;
        this.fechahora_sms = fechahora_sms;
        this.estado_pozo = estado_pozo;
        this.info_alarma = info_alarma;
    }

    //Getters & Setters - Necesarios para el POJO

    public int getId_pozo() {
        return id_pozo;
    }

    public void setId_pozo(int id_pozo) {
        this.id_pozo = id_pozo;
    }

    public int getId_riego() {
        return id_riego;
    }

    public void setId_riego(int id_riego) {
        this.id_riego = id_riego;
    }

    public int getId_alarma() {
        return id_alarma;
    }

    public void setId_alarma(int id_alarma) {
        this.id_alarma = id_alarma;
    }

    public long getFecha_hora() {
        return fecha_hora;
    }

    public void setFecha_hora(long fecha_hora) {
        this.fecha_hora = fecha_hora;
    }

    public String getFecha_inicio() {
        return fecha_inicio;
    }

    public void setFecha_inicio(String fecha_inicio) {
        this.fecha_inicio = fecha_inicio;
    }

    public String getHora_inicio() {
        return hora_inicio;
    }

    public void setHora_inicio(String hora_inicio) {
        this.hora_inicio = hora_inicio;
    }

    public int getDuracion() {
        return duracion;
    }

    public void setDuracion(int duracion) {
        this.duracion = duracion;
    }

    public long getFechahora_sms() {
        return fechahora_sms;
    }

    public void setFechahora_sms(long fechahora_sms) {
        this.fechahora_sms = fechahora_sms;
    }

    public int getEstado_pozo() {
        return estado_pozo;
    }

    public void setEstado_pozo(int estado_pozo) {
        this.estado_pozo = estado_pozo;
    }

    public int getInfo_alarma() {
        return info_alarma;
    }

    public void setInfo_alarma(int info_alarma) {
        this.info_alarma = info_alarma;
    }


    public ETipoEstado getEnum_estadopozo() {
        return ETipoEstado.values()[estado_pozo];
    }

    public ETipoAlarma getEnum_infoalarma() {
        return ETipoAlarma.values()[info_alarma];
    }

    @Override
    public String toString() {
        return "InfoAlarmaHist{" +
                "id_pozo=" + id_pozo +
                ", id_riego=" + id_riego +
                ", id_alarma=" + id_alarma +
                ", fecha_hora=" + fecha_hora +
                ", fecha_inicio='" + fecha_inicio +
                ", hora_inicio='" + hora_inicio +
                ", duracion=" + duracion +
                ", fechahora_sms=" + fechahora_sms +
                ", estado_pozo=" + getEnum_estadopozo().getTexto() +
                "info_alarma=" + getEnum_infoalarma().getTexto() +
                '}';
    }

    public String conversionDuracion(){
        int duracion_min;
        int duracion_horas;
        String horas;
        String minutos;
        String conversion;

        duracion_horas = getDuracion()/60;
        duracion_min =  getDuracion()- (duracion_horas * 60);
        minutos = Integer.toString(duracion_min);
        if (duracion_min < 10) {
            minutos = "0" + minutos;
        }
        horas = Integer.toString(duracion_horas);
        conversion = horas + "h " + minutos + "m";
        return conversion;
    }

    //Obtener Recurso mipmap asociado al Estado del pozo
    public int getRecursoIconoEstado (){
        int id;
        int idestado = getEnum_estadopozo().getRecurso();

        //Log.d("===> estado ",Integer.toString(idestado));
        switch(idestado) {
            case 0:
                id = R.mipmap.ic_alarm_red;
                break;
            case 1:
                id = R.mipmap.ic_alarm_red;
                break;
            case 2:
                id = R.mipmap.ic_alarm_red;
                break;
            case 3:
                id = R.mipmap.ic_alarm_red;
                break;
            case 4:
                id = R.mipmap.ic_alarm_red;
                break;
            case 5:
                id = R.mipmap.ic_alarm_red;
                break;
            case 6:
                id = R.mipmap.ic_alarm_red;
                break;
            case 7:
                id = R.mipmap.ic_alarm_blue;
                break;
            case 8:
                id = R.mipmap.ic_alarm_orange;
                break;
            case 9:
                id = R.mipmap.ic_alarm_grey;
                break;
            default:
                id = R.mipmap.ic_alarm_grey;
                break;

        }
        //Log.d("===> id icono ",Integer.toString(id));
        return id;
    }

}
