package com.examples.pfsoft.mispozosderiego;


import android.util.Log;

/**
 * Created by VRaul on 05/12/2016 - @PFSoft 2016
 * Clase que gestiona la Información básica asociada al Riego
 * La heredan otras clases
 */

public class InfoBasicaRiego {

    private int id_pozo; //Clave relacional para enlaza con la información de Tabla Pozos
    private long fecha_hora;  //Fecha y Hora Registro en BD en milisegundos  [Formato visualización:dd/mm/aaaa - hh:mm]
    private String fecha_inicio; //Fecha Inicio Riego o Alarma [Formato visualización:dd/mm/aaaa]
    private String hora_inicio;  //Hora Inicio Riego o Alarma [Formato visualización:hh:mm]
    private int programa; //Indica Número del Programa de Riego utilizado. (Valores: 1 .. 10, Valor 0: indica riego manual)
    private int sectores; //Codifica en un entero los sectores que se están regando
    private int duracion; //Duración  actual del riego (expresado en minutos)  [Formato visualización:hh:mm]
    private float caudal; //Caudal de agua que lleva consumido en el Riego (m³) [Formato visualización: xx.x]
    private float pot_consumida; //Potencia eléctrica que lleva consumida en el Riego (kWh)  [Formato visualización: xx.x]
    private int id_riego; //Clave relacional con Tabla Historico Riegos (0: Sin riego asociado)

    //Constructores
    public InfoBasicaRiego(){
        id_pozo = 0;
        fecha_hora= System.currentTimeMillis();
        id_riego = 0;
    }

    public InfoBasicaRiego(int id_pozo){
        this.id_pozo = id_pozo;
        fecha_hora = System.currentTimeMillis();
        fecha_inicio = "";
        hora_inicio = "";
        programa = 0;
        sectores = 0;
        duracion = 0;
        caudal = 0;
        pot_consumida = 0;
        id_riego = 0;
    }

    public InfoBasicaRiego(int id_pozo, long fecha_hora, String fecha_inicio, String hora_inicio) {
        this.id_pozo = id_pozo;
        this.fecha_hora = fecha_hora;
        this.fecha_inicio = fecha_inicio;
        this.hora_inicio = hora_inicio;

    }

    //Getters & Setters - Necesarios para el POJO

    public int getId_pozo() {
        return id_pozo;
    }

    public void setId_pozo(int id_pozo) {
        this.id_pozo = id_pozo;
    }

    public long getFecha_hora() {
        return fecha_hora;
    }

    public void setFecha_hora(long fecha_hora) {
        this.fecha_hora = fecha_hora;
    }

    public String getFecha_inicio() {
        return fecha_inicio;
    }

    public void setFecha_inicio(String fecha_inicio) {
        this.fecha_inicio = fecha_inicio;
    }

    public String getHora_inicio() {
        return hora_inicio;
    }

    public void setHora_inicio(String hora_inicio) {
        this.hora_inicio = hora_inicio;
    }

    public int getPrograma() {
        return programa;
    }

    public void setPrograma(int programa) {
        this.programa = programa;
    }

    public int getSectores() {
        return sectores;
    }

    public void setSectores(int sectores) {
        this.sectores = sectores;
    }

    public int getDuracion() {
        return duracion;
    }

    public void setDuracion(int duracion) {
        this.duracion = duracion;
    }

    public float getCaudal() {
        return caudal;
    }

    public void setCaudal(float caudal) {
        this.caudal = caudal;
    }

    public float getPot_consumida() {
        return pot_consumida;
    }

    public void setPot_consumida(float pot_consumida) {
        this.pot_consumida = pot_consumida;
    }

    public int getId_riego() {
        return id_riego;
    }

    public void setId_riego(int id_riego) {
        this.id_riego = id_riego;
    }

    @Override
    public String toString() {
        return "InfoEstadoPozo{" +
                "id_pozo=" + id_pozo +
                ", fecha_hora=" + fecha_hora +
                ", fecha_inicio='" + fecha_inicio + '\'' +
                ", hora_inicio='" + hora_inicio + '\'' +
                ", programa=" + programa +
                ", sectores=" + sectores +
                ", duracion=" + duracion +
                ", caudal=" + caudal +
                ", pot_consumida=" + pot_consumida +
                ", id_riego=" + id_riego +
                '}';
    }

    public boolean esSectorActivo(int sector){
        int sectoresActivos;
        long mascara;
        long aux;
        boolean esActivo;

        sectoresActivos = getSectores();
        mascara = Math.round(Math.pow(2, (sector - 1)));
        //Log.d("Mascara",Long.toString(mascara));
        aux = sectoresActivos & mascara;
        //Log.d("Auxiliar",Long.toString(aux));
        esActivo = (aux == mascara);
        //Log.d("Es Activo Sector ",Integer.toString(sector)+": " + Boolean.toString(esActivo));
      return  esActivo;
    }

    public String conversionDuracion(){
        int duracion_min;
        int duracion_horas;
        String horas;
        String minutos;
        String conversion;

        duracion_horas = getDuracion()/60;
        duracion_min =  getDuracion()- (duracion_horas * 60);
        minutos = Integer.toString(duracion_min);
        if (duracion_min < 10) {
            minutos = "0" + minutos;
        }
        horas = Integer.toString(duracion_horas);
        conversion = horas + "h " + minutos + "m";
        return conversion;
    }
}
