package com.examples.pfsoft.mispozosderiego;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.os.IBinder;
import android.os.SystemClock;
import android.support.v4.app.NotificationCompat;
import android.util.Log;
import android.widget.Toast;

import com.google.firebase.database.DatabaseReference;

/**
 * Created by VRaul on 14/12/2016.  - [Revision @PFSoft 09/2017]
 * Clase implementa Servicio  --> Encargado de procesar el SMS recibido
 */


public class ServicioProcesarSMS extends Service {

    private static final int ID_NOTIFICACION_CREAR = 1;
    private String sms_telefono;
    private String sms_mensaje;
    private Aplicacion app;

    @Override
    public void onCreate() {
        app = (Aplicacion) getApplication();
        Log.d("[ServicioProcesarSMS] ", "Servicio Procesamiento SMS creado...");
    }

    @Override
    public int onStartCommand(Intent intencion, int flags, int idArranque) {

        //Extrae información pasada desde el receptor de anuncios
        Bundle extras = intencion.getExtras();
        sms_telefono = extras.getString("smsnumber");
        sms_mensaje = extras.getString("smsbodytext");

        //Comenzar procesamiento información SMS
        ProcesarSMS(idArranque);

        return START_STICKY;



    }

    @Override
    public void onDestroy() {
        //Cuando se detiene el Servicio

        Log.d("[ServicioProcesarSMS] ", "Servicio Procesamiento SMS detenido...");
    }

    @Override
    public IBinder onBind(Intent intencion) {
        return null;
    }

    public void ProcesarSMS(int idArranque){

        Log.d("[ProcesarSMS]", "SMS Recibido- Telefono: " + sms_telefono + " - Mensaje: " + sms_mensaje);

        //Solo Pruebas - Eliminar de la versión final
        SystemClock.sleep(1500);

        //=====Código procesamiento del mensaje y del Telefono =========

        boolean sms_valid = false;
        boolean sms_alarma = false;

        InfoEstadoPozo estado_pozo = new InfoEstadoPozo();
        InfoAlarmaHist infoalarma = new InfoAlarmaHist();

        //1. Verifica que SMS contine INFO y separadores ","
        if ((sms_mensaje.contains("INFO")) && (sms_mensaje.contains(","))){
            //2.- Extraer los items del mensaje por sus separadores ","
            String [] sms_items = sms_mensaje.split(",");
            //3.- Verificar numero total de items extraidos del sms
            // Caso 1: SMS InfoEstado sin InfoAlarma adicional...
            if (sms_items.length == GLOBALCONST.MAX_ITEMS_SMSINFO){
                Log.d("SMS_items ", Integer.toString(sms_items.length));
                //4.- Verificar validez de cada uno de los items para su posterior asignaión
                if (sms_items[0].contentEquals("INFO")) {

                    sms_valid = ExtraerSMS_InfoEstado(sms_items, estado_pozo);
                    if (sms_valid) {
                        Log.d("ProcesarSMS ", "Formato VALIDO...");
                        Log.d("===> estado_pozo ", estado_pozo.toString());
                        //Solo Pruebas - Eliminar de la versión final
                        //Toast.makeText(this, "[Pruebas] SMS Procesado: CON Info de Estado Riego", Toast.LENGTH_LONG).show();
                    } else {
                        Log.d("ProcesarSMS ", "Formato NO VALIDO...");
                        //Solo Pruebas - Eliminar de la versión final
                        //Toast.makeText(this, "[Pruebas] SMS Procesado: SIN Info de Estado Riego", Toast.LENGTH_LONG).show();
                    }
                }

            }

            //Caso 2: SMS InfoEstado con InfoAlarma adicional
            if ((sms_items.length == GLOBALCONST.MAX_ITEMS_SMSALM)) {
                Log.d("SMS_items ", Integer.toString(sms_items.length));
                //4.- Verificar validez de cada uno de los items para su posterior asignaión
                if (sms_items[0].contentEquals("INFO")) {
                    sms_valid = ExtraerSMS_InfoEstado(sms_items, estado_pozo);
                    if (sms_valid) {
                        sms_alarma = ExtraerSMS_InfoAlarma(sms_items, estado_pozo, infoalarma);
                        Log.d("ProcesarSMS ", "InfoEstado Formato VALIDO");
                        Log.d("===>estado_pozo ", estado_pozo.toString());
                        //Solo Pruebas - Eliminar de la versión final
                        //Toast.makeText(this, "[Pruebas] SMS Procesado: Parte 1 - CON Info de Estado Riego", Toast.LENGTH_LONG).show();
                    } else {
                        Log.d("ProcesarSMS ", "InfoEstado Formato NO VALIDO");
                        //Solo Pruebas - Eliminar de la versión final
                        //Toast.makeText(this, "[Pruebas] SMS Procesado: Parte 1 - SIN Info de Estado Riego", Toast.LENGTH_LONG).show();
                    }
                    if (sms_alarma)  {
                        Log.d("ProcesarSMS ", "InfoAlarma Formato VALIDO");
                        Log.d("===>info_alarma ", infoalarma.toString());
                        //Solo Pruebas - Eliminar de la versión final
                        //Toast.makeText(this, "[Pruebas] SMS Procesado: Parte 2 - CON Info de Alarma Riego", Toast.LENGTH_LONG).show();
                    } else {
                        Log.d("ProcesarSMS ", "InfoAlarma Formato NO VALIDO");
                        //Solo Pruebas - Eliminar de la versión final
                        //Toast.makeText(this, "[Pruebas] SMS Procesado: Parte 2 - SIN Info de Alarma Riego", Toast.LENGTH_LONG).show();
                    }
                }
            }

        }

        //Obtener info pozo a partir ID_POZO para generar información de la Notificación
        if (sms_valid) {
            String keyidpozo = app.getStrKeyIdPozo(estado_pozo.getId_pozo());
            InfoPozoRiego pozo = app.getPozos().elementoByKey(keyidpozo);

            //1.- Crea una Notificación si se ha procesado correcamente SMS relacionado con un Pozo de Riego
            if (pozo != null) {
                int idres = estado_pozo.getRecursoIconoEstado();

                Log.d("[Notificacion] IDPOZO", Integer.toString(pozo.getId_pozo()));

                NotificationCompat.Builder notific = new NotificationCompat.Builder(this)
                        .setLargeIcon(BitmapFactory.decodeResource(getResources(), idres))
                        .setWhen(System.currentTimeMillis() + 1000 * 60 * 60)
                        .setContentInfo(sms_telefono)
                        .setTicker("Por implementar...")
                        .setContentTitle(pozo.getNombre())
                        .setSmallIcon(R.mipmap.ic_pozoriego)
                        .setContentText(estado_pozo.getEnum_estadopozo().getTexto())
                        .setDefaults(Notification.DEFAULT_SOUND);

                //Al pulsar sobre la Notificación se ira a la Pantalla de Vista Info Estado... (Falta implementar)
                //Ojo!!! es importante ponerlo antes de hacer notific,build()
                Intent i = new Intent(this, ActivityMain.class);
                //i.putExtra("id", (long) pozo.getId_pozo());
                PendingIntent intencionPendiente = PendingIntent.getActivity(this, 0, i, 0);
                notific.setContentIntent(intencionPendiente);

                NotificationManager notificationManager = (NotificationManager)
                        getSystemService(Context.NOTIFICATION_SERVICE);
                notificationManager.notify(ID_NOTIFICACION_CREAR, notific.build());


                //2.- Actualizar nodo (keyidpozo) en tabla estadopozo de FireBase...
                app.getEstadopozos().actualizaByKey(keyidpozo, estado_pozo);
                Log.d("===>[Firebase]", "Actualiza Nodo estadopozo[" + keyidpozo + "] " );

                //3.- Si es necesario: Actualizar nodo (keyidpozo --> keyidriego) en tabla riegos
                if (EsActualizaInfoRiego(estado_pozo)) {
                    InfoBasicaRiego inforiego = new InfoBasicaRiego();
                    inforiego = (InfoBasicaRiego) estado_pozo;
                    String keyidriego = app.getStrKeyIdRiego(estado_pozo.getId_riego());
                    //Obtiene la referencia al riego y actualiza su contenido
                    DatabaseReference riegoRef = app.getRiegosReference().child(keyidpozo).child(keyidriego);
                    riegoRef.setValue(inforiego);
                    Log.d("===>[Firebase]", "Actualiza Nodo riego[" + keyidpozo + "][" + keyidriego + "] ");
                }

                //4.- Si es necesario: Actualizar nodo (keyidpozo-->keyidalarma) en tabla alarmas
                //   POR IMPLEMENTAR!!!
                if (sms_alarma) {
                    if (EsActualizaInfoAlarma(estado_pozo)) {
                        String keyidalarma = app.getStrKeyIdAlarma(infoalarma.getId_alarma());
                        //Obtiene la referencia al riego y actualiza su contenido
                        DatabaseReference alarmaRef = app.getAlarmasReference().child(keyidpozo).child(keyidalarma);
                        alarmaRef.setValue(infoalarma);
                        Log.d("===>[Firebase]", "Actualiza Nodo alarma[" + keyidpozo + "][" + keyidalarma + "] ");
                    }
                }


            }

        }

        if (!sms_valid) {
            //Solo Pruebas - Eliminar de la versión final
            Log.d("ProcesarSMS ", "SIN Info de Estado Riego...");
            //Toast.makeText(this, "[Pruebas] SMS Procesado: SIN Info de Estado Riego", Toast.LENGTH_LONG).show();
        }

        //Al finalizar detiene el Servicio
        stopSelf(idArranque);
    }


    private boolean ExtraerSMS_InfoEstado(String [] sms_items, InfoEstadoPozo estado_pozo){

        int valorInt;
        float valorFloat;

        Log.d("1.- SMS_IDPOZO ", sms_items[1]);
        try {
            valorInt = Integer.valueOf(sms_items[1]);
        } catch (Exception e) {
            Log.d("IDPOZO Error", e.toString());
            return false;
        }
        if ((valorInt > 0) && (valorInt <= GLOBALCONST.MAX_IDPOZO)) {
            estado_pozo.setId_pozo(valorInt);
        } else {
            Log.d("IDPOZO Error", "Fuera del Rango permitido");
            return false;
        }

        Log.d("2.- SMS_PROGRAMA ", sms_items[2]);
        try {
            valorInt = Integer.valueOf(sms_items[2]);
        } catch (Exception e) {
            Log.d("PROGRAMA Error", e.toString());
            return false;
        }
        if ((valorInt >= 0) && (valorInt <= GLOBALCONST.MAX_PROGRAMAS)) {
            estado_pozo.setPrograma(valorInt);
        } else {
            Log.d("PROGRAMA Error", "Fuera del Rango permitido");
            return false;
        }

        Log.d("3.- SMS_IDRIEGO ", sms_items[3]);
        try {
            valorInt = Integer.valueOf(sms_items[3]);
        } catch (Exception e) {
            Log.d("IDRIEGO Error", e.toString());
            return false;
        }
        if ((valorInt >= 0) && (valorInt <= GLOBALCONST.MAX_IDRIEGO)){
            estado_pozo.setId_riego(valorInt);
        } else {
            Log.d("IDRIEGO Error", "Fuera del Rango permitido");
            return false;
        }

        Log.d("4.- SMS_ESTADO ", sms_items[4]);
        try {
            valorInt = Integer.valueOf(sms_items[4]);
        } catch (Exception e) {
            Log.d("ESTADO Error", e.toString());
            return false;
        }
        estado_pozo.setEstado_pozo(valorInt);

        Log.d("5.- SMS_ALARMA ", sms_items[5]);
        try {
            valorInt = Integer.valueOf(sms_items[5]);
        } catch (Exception e) {
            Log.d("ALARMA Error", e.toString());
            return false;
        }
        estado_pozo.setInfo_alarma(valorInt);

        Log.d("6.- SMS_FINICIO ", sms_items[6]);
        estado_pozo.setFecha_inicio(sms_items[6]);

        Log.d("7.- SMS_HINICIO ", sms_items[7]);
        estado_pozo.setHora_inicio(sms_items[7]);

        Log.d("8.- SMS_DURACION ", sms_items[8]);
        try {
            valorInt = Integer.valueOf(sms_items[8]);
        } catch (Exception e) {
            Log.d("DURACION Error", e.toString());
            return false;
        }
        estado_pozo.setDuracion(valorInt);

        Log.d("9.- SMS_SECTORES ", sms_items[9]);
        try {
            valorInt = Integer.valueOf(sms_items[9]);
        } catch (Exception e) {
            Log.d("SECTORES Error", e.toString());
            return false;
        }
        double maxcod = Math.pow(2, GLOBALCONST.MAX_SECTORES) - 1;
        if ((valorInt >= 0) && (valorInt <= maxcod)){
            estado_pozo.setSectores(valorInt);
        } else {
            Log.d("SECTORES Error", "Fuera del Rango permitido - MAXCOD: " + Double.toString(maxcod));
            return false;
        }

        Log.d("10.- SMS_CAUDAL ", sms_items[10]);
        try {
            valorFloat = Float.valueOf(sms_items[10]);
        } catch (Exception e) {
            Log.d("CAUDAL Error", e.toString());
            return false;
        }
        estado_pozo.setCaudal(valorFloat);

        Log.d("11.- SMS_POT ", sms_items[11]);
        try {
            valorFloat = Float.valueOf(sms_items[11]);
        } catch (Exception e) {
            Log.d("POT Error", e.toString());
            return false;
        }
        estado_pozo.setPot_consumida(valorFloat);

        return true;
    }

    private boolean ExtraerSMS_InfoAlarma(String [] sms_items, InfoEstadoPozo estado_pozo, InfoAlarmaHist infoalarma){

        int valorInt;

        infoalarma.setId_pozo(estado_pozo.getId_pozo());
        infoalarma.setId_riego(estado_pozo.getId_riego());
        infoalarma.setInfo_alarma(estado_pozo.getInfo_alarma());
        infoalarma.setFechahora_sms(estado_pozo.getFechahora_sms());
        infoalarma.setEstado_pozo(estado_pozo.getEstado_pozo());

        Log.d("12.- SMS_ID_ALARMA ", sms_items[12]);
        try {
            valorInt = Integer.valueOf(sms_items[12]);
        } catch (Exception e) {
            Log.d("ID_ALARMA Error", e.toString());
            return false;
        }
        if ((valorInt > 0) && (valorInt <= GLOBALCONST.MAX_IDALARMA)) {
            infoalarma.setId_alarma(valorInt);
        } else {
            Log.d("ID_ALARMA Error", "Fuera del Rango permitido");
            return false;
        }

       Log.d("13.- SMS_FINICIO_ALM ", sms_items[13]);
        infoalarma.setFecha_inicio(sms_items[13]);

        Log.d("14.- SMS_HINICIO_ALM ", sms_items[14]);
        infoalarma.setHora_inicio(sms_items[14]);

        Log.d("15.- SMS_DURACION_ALM ", sms_items[15]);
        try {
            valorInt = Integer.valueOf(sms_items[15]);
        } catch (Exception e) {
            Log.d("DURACION_ALM Error", e.toString());
            return false;
        }
        infoalarma.setDuracion(valorInt);

        return true;
    }

    //Determina si hay que Actualizar nodo InfoRiego asociada clave ID_POZO y subclave ID_RIEGO
    //en la tabla riegos.
    private boolean EsActualizaInfoRiego(InfoEstadoPozo estado_pozo){
        boolean testresult = ((estado_pozo.getEnum_estadopozo()== ETipoEstado.RIEGO_MAN) ||
                (estado_pozo.getEnum_estadopozo()== ETipoEstado.RIEGO_PROG) ||
                (estado_pozo.getEnum_estadopozo()== ETipoEstado.FIN_RIEGO_MAN) ||
                (estado_pozo.getEnum_estadopozo()== ETipoEstado.FIN_RIEGO_PROG) ||
                (estado_pozo.getEnum_estadopozo()== ETipoEstado.ALM_RIEGO_MAN) ||
                (estado_pozo.getEnum_estadopozo()== ETipoEstado.ALM_RIEGO_PROG));


     return testresult;
    }

    //Determina si hay que Actualizar nodo InfoAlarma asociada clave ID_POZO y subclave ID_RIEGO
    //en la tabla alarmas.
    private boolean EsActualizaInfoAlarma(InfoEstadoPozo estado_pozo){
        boolean testresult = ((estado_pozo.getEnum_estadopozo()== ETipoEstado.ALARMA) ||
                (estado_pozo.getEnum_estadopozo()== ETipoEstado.ALM_RIEGO_MAN) ||
                (estado_pozo.getEnum_estadopozo()== ETipoEstado.ALM_RIEGO_PROG) ||
                (estado_pozo.getEnum_estadopozo()== ETipoEstado.FIN_ALARMA) ||
                (estado_pozo.getEnum_estadopozo()== ETipoEstado.FINALM_RIEGO_MAN) ||
                (estado_pozo.getEnum_estadopozo()== ETipoEstado.FINALM_RIEGO_PROG));


        return testresult;
    }






}
